import { getInspectorControlsAttributes } from '../../../../../../../grimlock/assets/js/block-editor/src/block/utils';
import Save from '../../../../../../../grimlock/assets/js/block-editor/src/block/wrapper/save';
import getWrapperBlockProps from '../../../../../../../grimlock/assets/js/block-editor/src/block/wrapper/props';
import wrapperAttributes from '../../../../../../../grimlock/assets/js/block-editor/src/block/wrapper/attributes';

export const inspectorControls = {
	darkMode: {
		label: 'Dark Mode',
		fields: [
			{
				type: 'colorSettings',
				name: 'darkModeBackgroundColor',
				label: 'Background',
				settings: [ {
					label: 'Background Color',
					name: 'background',
					enableGradient: true,
				}, {
					label: 'Overlay Color',
					name: 'overlay',
					enableGradient: true,
				} ],
				default: { background: '', overlay: '' },
				attributeType: 'object',
			},
			{
				type: 'colorSettings',
				name: 'darkModeTextColor',
				label: 'Text',
				settings: [ {
					label: 'Headings Color',
					name: 'headings',
					enableGradient: true,
				}, {
					label: 'Text Color',
					name: 'text',
					enableGradient: true,
				} ],
				default: { text: '', headings: '' },
				attributeType: 'object',
			},
			{
				type: 'checkbox',
				name: 'darkModeForceHeadingsColor',
				label: 'Force headings color inside this wrapper in dark mode',
				default: false,
				attributeType: 'boolean',
			},
			{
				type: 'checkbox',
				name: 'darkModeForceTextColor',
				label: 'Force text color inside this wrapper in dark mode',
				default: false,
				attributeType: 'boolean',
			},
			{
				type: 'colorSettings',
				name: 'darkModeBorderColor',
				label: 'Border',
				settings: [ {
					label: 'Top Border Color',
					name: 'top',
					enableGradient: false,
				}, {
					label: 'Right Border Color',
					name: 'right',
					enableGradient: false,
				}, {
					label: 'Bottom Border Color',
					name: 'bottom',
					enableGradient: false,
				}, {
					label: 'Left Border Color',
					name: 'left',
					enableGradient: false,
				} ],
				default: { text: '', headings: '' },
				attributeType: 'object',
			},
		],
	}
};

const attributes = getInspectorControlsAttributes( inspectorControls );

export const getBlockProps = ( blockProps, attributes ) => {
	if ( attributes.darkModeForceHeadingsColor )
		blockProps.className += ' grimlock-dark-mode--force-heading-color';

	if ( attributes.darkModeForceTextColor )
		blockProps.className += ' grimlock-dark-mode--force-text-color';

	blockProps.style = {
		...blockProps.style,
		'--grimlock-dark-mode-wrapper-headings-color': attributes?.darkModeTextColor?.headings || null,
		'--grimlock-dark-mode-wrapper-text-color': attributes?.darkModeTextColor?.text || null,
		'--grimlock-dark-mode-wrapper-background-color': attributes.darkModeBackgroundColor?.background || null,
		'--grimlock-dark-mode-wrapper-background-overlay-color': attributes.darkModeBackgroundColor?.overlay || null,
		'--grimlock-dark-mode-wrapper-border-top-color': attributes.darkModeBorderColor?.top || null,
		'--grimlock-dark-mode-wrapper-border-right-color': attributes.darkModeBorderColor?.right || null,
		'--grimlock-dark-mode-wrapper-border-bottom-color': attributes.darkModeBorderColor?.bottom || null,
		'--grimlock-dark-mode-wrapper-border-left-color': attributes.darkModeBorderColor?.left || null,
	};

	return blockProps;
};

export const deprecator = {
	attributes: {
		...wrapperAttributes,
		...attributes,
	},
	save: ( props ) => <Save { ...{ blockProps: getBlockProps( getWrapperBlockProps( props.attributes, true ), props.attributes ), ...props } } />,
};
