<?php
/*
 * Plugin name: Grimlock Dark Mode
 * Plugin URI:  https://www.themosaurus.com/
 * Description: This plugin allows you to setup a dark mode for your website
 * Author:      Themosaurus
 * Author URI:  https://www.themosaurus.com/
 * Version:     1.0.7
 * License:     GPL-2.0+
 * License URI: http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain: grimlock-dark-mode
 * Domain Path: /languages
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

define( 'GRIMLOCK_DARK_MODE_VERSION',              '1.0.7' );
define( 'GRIMLOCK_DARK_MODE_MIN_GRIMLOCK_VERSION', '1.5.5' );
define( 'GRIMLOCK_DARK_MODE_MIN_KIRKI_VERSION',    '4.0.0' );
define( 'GRIMLOCK_DARK_MODE_PLUGIN_FILE',          __FILE__ );
define( 'GRIMLOCK_DARK_MODE_PLUGIN_DIR_PATH',      plugin_dir_path( __FILE__ ) );
define( 'GRIMLOCK_DARK_MODE_PLUGIN_DIR_URL',       plugin_dir_url( __FILE__ ) );

// Initialize update checker
require 'libs/plugin-update-checker/plugin-update-checker.php';
Puc_v4_Factory::buildUpdateChecker(
	'http://files.themosaurus.com/grimlock-dark-mode/version.json',
	__FILE__,
	'grimlock-dark-mode'
);

/**
 * Display notice if Grimlock version doesn't match minimum requirement
 */
function grimlock_dark_mode_dependency_notice() {
	$url = self_admin_url( 'update-core.php?action=do-plugin-upgrade&plugins=' ) . urlencode( plugin_basename( GRIMLOCK_PLUGIN_FILE ) );
	$url = wp_nonce_url( $url, 'upgrade-core' ); ?>
	<div class="notice notice-error">
		<p><?php printf( esc_html__( '%1$sGrimlock Dark Mode%2$s requires %1$sGrimlock %3$s%2$s to continue running properly. Please %4$supdate Grimlock.%5$s', 'grimlock-dark-mode' ), '<strong>', '</strong>', GRIMLOCK_DARK_MODE_MIN_GRIMLOCK_VERSION, '<a href="' . esc_url( $url ) . '">', "</a>" ); ?></p>
	</div>
	<?php
}

/**
 * Display notice if Kirki version doesn't match minimum requirement
 */
function grimlock_dark_mode_kirki_dependency_notice() {
	$url = self_admin_url( 'update-core.php?action=do-plugin-upgrade&plugins=' ) . urlencode( plugin_basename( KIRKI_PLUGIN_FILE ) );
	$url = wp_nonce_url( $url, 'upgrade-core' ); ?>
	<div class="notice notice-error">
		<p><?php printf( esc_html__( '%1$sGrimlock Dark Mode%2$s requires %1$sKirki %3$s%2$s to continue running properly. Please %4$supdate Kirki.%5$s', 'grimlock-dark-mode' ), '<strong>', '</strong>', GRIMLOCK_DARK_MODE_MIN_KIRKI_VERSION, '<a href="' . esc_url( $url ) . '">', "</a>" ); ?></p>
	</div>
	<?php
}

/**
 * Load plugin.
 */
function grimlock_dark_mode_loaded() {
	if ( version_compare( GRIMLOCK_VERSION, GRIMLOCK_DARK_MODE_MIN_GRIMLOCK_VERSION, '<' ) ) {
		add_action( 'admin_notices', 'grimlock_dark_mode_dependency_notice' );
		return;
	}

	if ( version_compare( KIRKI_VERSION, GRIMLOCK_DARK_MODE_MIN_KIRKI_VERSION, '<' ) ) {
		add_action( 'admin_notices', 'grimlock_dark_mode_kirki_dependency_notice' );
		return;
	}

	require_once 'inc/class-grimlock-dark-mode.php';

	global $grimlock_dark_mode;
	$grimlock_dark_mode = new Grimlock_Dark_Mode();

	do_action( 'grimlock_dark_mode_loaded' );
}
add_action( 'grimlock_loaded', 'grimlock_dark_mode_loaded' );
