<?php
// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Grimlock_Dark_Mode_Blocks
 *
 * @author  themosaurus
 * @since   1.0.0
 * @package grimlock-dark-mode/inc
 */
class Grimlock_Dark_Mode_Blocks {

	/**
	 * Setup class.
	 */
	public function __construct() {
		add_action( 'init', array( $this, 'generate_dark_mode_fields' ), 2000 );
	}

	/**
	 * Generate Dark Mode panel and fields in blocks
	 */
	public function generate_dark_mode_fields() {
		$blocks_data = apply_filters( 'grimlock_blocks_js_data', array( 'blocks' => array() ) );

		foreach ( $blocks_data['blocks'] as $block_name => $block ) {
			if ( ! empty( $block['panels'] ) ) { // Back-compat with old blocks js data TODO: remove when all blocks use new structure
				$block['inspectorControls'] = $block['panels'];
				$block_name = $block['name'];
			}

			if ( empty( $block['inspectorControls'] ) ) {
				continue;
			}

			$block_type = unregister_block_type( $block_name );
			if ( empty( $block_type ) ) {
				continue;
			}

			$split_block_name = explode( '/', $block_name );
			$block_id_base    = str_replace( '-', '_', $split_block_name[0] . '_' . $split_block_name[1] . '_block' );

			$dark_mode_fields = array();

			foreach ( $block['inspectorControls'] as $panel ) {
				if ( empty( $panel['fields'] ) ) {
					continue;
				}

				foreach ( $panel['fields'] as $field ) {

					// Background gradient displayed toggle
					if ( isset( $field['name'] ) && $field['name'] === 'background_gradient_displayed' ) {
						$new_name = "dark_mode_{$field['name']}";

						$dark_mode_fields[] = array_merge( $field, array(
							'name' => $new_name,
						) );

						$block_type->attributes[ $new_name ] = array(
							'type'    => $field['attribute_type'],
							'default' => false,
						);

						continue;
					}

					// Bail if not a color or gradient field
					if ( ! in_array( $field['type'], array( 'color', 'gradient' ) ) ) {
						continue;
					}

					$new_name  = "dark_mode_{$field['name']}";
					$new_field = array_merge( $field, array( 'name' => $new_name ) );

					// Adapt conditional logic for background gradient
					if ( $field['name'] === 'background_gradient' ) {
						$new_field['conditional_logic'][0]['field'] = 'dark_mode_background_gradient_displayed';
					}

					$dark_mode_fields[] = $new_field;

					$block_type->attributes[ $new_name ] = array(
						'type'    => $field['attribute_type'],
						'default' => '',
					);

					add_filter( "{$block_id_base}_component_args", function( $args, $attributes ) use( $field, $split_block_name ) {
						if ( ! empty( $attributes[ "dark_mode_{$field['name']}" ] ) ) {
							$css_var_prefix = strpos( $split_block_name[1], 'section' ) !== false ? '--grimlock-dark-mode-section-' : "--grimlock-dark-mode-{$split_block_name[1]}-";
							$css_var        = $css_var_prefix . str_replace( '_', '-', $field['name'] );

							$args['styles'] = array_merge( $args['styles'], array(
								$css_var => $attributes["dark_mode_{$field['name']}"],
							) );
						}

						return $args;
					}, 100, 2 );
				}
			}

			if ( ! empty( $dark_mode_fields ) ) {

				if ( ! empty( $block['panels'] ) ) { // Back-compat with old blocks js data TODO: remove when all blocks use new structure
					$blocks_data['blocks'][ $block_id_base ]['panels']['dark-mode'] = array(
						'label'  => esc_html__( 'Dark Mode', 'grimlock-dark-mode' ),
						'fields' => $dark_mode_fields,
					);
				}
				else {
					$blocks_data['blocks'][ $block_name ]['inspectorControls']['dark-mode'] = array(
						'label'  => esc_html__( 'Dark Mode', 'grimlock-dark-mode' ),
						'fields' => $dark_mode_fields,
					);
				}

			}

			register_block_type( $block_type );
		}

		add_filter( 'grimlock_blocks_js_data', function( $original_blocks_data ) use( $blocks_data ) {
			return array_merge( $original_blocks_data, $blocks_data );
		}, 100 );
	}
}

return new Grimlock_Dark_Mode_Blocks();
