<?php
// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Grimlock_Dark_Mode
 *
 * @author  octopix
 * @since   1.0.0
 * @package grimlock-dark-mode/inc
 */
class Grimlock_Dark_Mode {
	/**
	 * Setup class.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		load_plugin_textdomain( 'grimlock-dark-mode', false, 'grimlock-dark-mode/languages' );

		add_action( 'wp_enqueue_scripts',          array( $this, 'enqueue_scripts'             ) );
		add_action( 'enqueue_block_editor_assets', array( $this, 'enqueue_block_editor_assets' ), 5 );

		// Shortcode
		add_shortcode( 'grimlock_dark_mode_toggle', array( $this, 'dark_mode_toggle_shortcode' ) );

		// Customizer
		require_once GRIMLOCK_DARK_MODE_PLUGIN_DIR_PATH . 'inc/customizer/class-grimlock-dark-mode-customizer.php';

		// Dynamic blocks
		require_once GRIMLOCK_DARK_MODE_PLUGIN_DIR_PATH . 'inc/block/class-grimlock-dark-mode-blocks.php';
	}

	/**
	 * Enqueue scripts and styles
	 */
	public function enqueue_scripts() {
		wp_enqueue_script( 'grimlock-dark-mode', GRIMLOCK_DARK_MODE_PLUGIN_DIR_URL . 'assets/js/main.js', array( 'jquery' ), GRIMLOCK_DARK_MODE_VERSION, true );
		wp_localize_script( 'grimlock-dark-mode', 'grimlockDarkMode', array( 'autodetect' => apply_filters( 'grimlock_dark_mode_default_mode', 'light' ) === 'autodetect' ) );
		wp_enqueue_style( 'grimlock-dark-mode', GRIMLOCK_DARK_MODE_PLUGIN_DIR_URL . 'assets/css/style.css', array(), GRIMLOCK_DARK_MODE_VERSION );
	}

	/**
	 * Enqueue scripts and stylesheets in the block editor
	 *
	 * @since 1.0.0
	 */
	public function enqueue_block_editor_assets() {
		// Automatically load script dependencies and version from webpack compiled file
		$asset_file = include( GRIMLOCK_DARK_MODE_PLUGIN_DIR_PATH . 'assets/js/block-editor/build/index.asset.php' );

		// Register block script
		wp_enqueue_script( 'grimlock-dark-mode-block-editor', GRIMLOCK_DARK_MODE_PLUGIN_DIR_URL . 'assets/js/block-editor/build/index.js', array_merge( $asset_file['dependencies'], array( 'wp-edit-post' ) ), GRIMLOCK_VERSION, true );
	}

	/**
	 * Dark mode toggle shortcode
	 * Usage: [grimlock_dark_mode_toggle]
	 *
	 * @return string
	 */
	public function dark_mode_toggle_shortcode() {
		ob_start(); ?>

		<button class="btn btn-secondary grimlock-dark-mode__toggle grimlock-dark-mode__toggle--shortcode"></button>

		<?php return ob_get_clean();
	}

}