<?php
/**
 * Grimlock_Dark_Mode_Customizer Class
 *
 * @author   Themosaurus
 * @since    1.3.19
 * @package grimlock
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * The Grimlock Customizer class for Dark_Mode.
 */
class Grimlock_Dark_Mode_Customizer extends Grimlock_Base_Customizer {

	private $panels = array();
	private $sections = array();
	private $fields = array();

	/**
	 * Setup class.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		$this->section = 'grimlock_dark_mode_section';
		$this->title   = esc_html__( 'Dark Mode', 'grimlock-dark-mode' );

		add_action( 'kirki_field_custom_init', array( $this, 'populate_fields'   ), 10, 3 );
		add_action( 'kirki_panel_added',       array( $this, 'populate_panels'   ), 10, 2 );
		add_action( 'kirki_section_added',     array( $this, 'populate_sections' ), 10, 2 );

		add_action( 'customize_register',                    array( $this, 'generate_dark_mode_customizer_hierarchy' ), 20     );
		add_action( 'after_setup_theme',                     array( $this, 'add_customizer_fields'                   ), 999    );

		add_filter( 'body_class',                            array( $this, 'add_body_class'                          ), 10     );
		add_action( 'wp_footer',                             array( $this, 'display_dark_mode_toggle'                )         );
		add_action( 'grimlock_navbar_nav_menu',              array( $this, 'navigation_display_dark_mode_toggle'     ), 200, 1 );
		add_action( 'grimlock_vertical_navbar_nav_menu',     array( $this, 'navigation_display_dark_mode_toggle'     ), 200, 1 );
		add_action( 'grimlock_hamburger_navbar_nav_menu',    array( $this, 'navigation_display_dark_mode_toggle'     ), 200, 1 );
		add_filter( 'grimlock_site_identity_args',           array( $this, 'change_site_identity_args'               )         );

		add_filter( 'grimlock_dark_mode_default_mode',       array( $this, 'default_mode'                             ), 10, 1 );
	}

	/**
	 * @param \Kirki\Field $field
	 * @param $args
	 * @param $control_class
	 */
	public function populate_fields( $field, $args, $control_class ) {
		if ( ! empty( $field->type ) ) {
			$this->fields[ $args['settings'] ] = array_merge( $args, array( 'field_class' => get_class( $field ) ) );
		}
	}

	public function populate_panels( $id, $args ) {
		$this->panels[ $id ] = $args;
	}

	public function populate_sections( $id, $args ) {
		$this->sections[ $id ] = $args;
	}

	/**
	 * Generate dark mode specific panels and fields in the customizer
	 */
	public function generate_dark_mode_customizer_hierarchy( $wp_customize ) {
		remove_action( 'kirki_panel_added',   array( $this, 'populate_panels'   ), 10 );
		remove_action( 'kirki_section_added', array( $this, 'populate_sections' ), 10 );

		$this->generate_dark_mode_panels( $wp_customize );
		$this->generate_dark_mode_sections( $wp_customize );
	}

	/**
	 * Generate Dark Mode panels based on existing Kirki panels
	 */
	protected function generate_dark_mode_panels( $wp_customize ) {
		if ( class_exists( 'Kirki' ) ) {
			// Create Dark Mode panel
			$panel = new \Kirki\Panel( 'grimlock_dark_mode_customizer_panel', array(
				'priority' => 999,
				'title'    => $this->title,
			) );
			$panel->add_panel( $wp_customize );

			// Generate Dark Mode sub-panels
			foreach ( $this->panels as $panel_id => $panel ) {
				$panel = new \Kirki\Panel( str_replace( 'grimlock_', 'grimlock_dark_mode_', $panel_id ), array_merge( $panel, array(
					'panel' => 'grimlock_dark_mode_customizer_panel',
				) ) );
				$panel->add_panel( $wp_customize );
			}
		}
	}

	/**
	 * Generate Dark Mode sections based on existing Kirki sections
	 */
	protected function generate_dark_mode_sections( $wp_customize ) {
		if ( class_exists( 'Kirki' ) ) {
			$section = new \Kirki\Section( $this->section, apply_filters( "{$this->section}_args", array(
				'title'    => esc_html__( 'Dark Mode Settings', 'grimlock-dark-mode' ),
				'priority' => 999,
				'panel'    => 'grimlock_dark_mode_customizer_panel',
			) ) );
			$section->add_section( $wp_customize );

			// Manually add "Site Identity" section because it is a default WP section (it doesn't exist in Kirki)
			$this->sections['title_tagline'] = array(
				'title'    => esc_html__( 'Site Identity', 'grimlock-dark-mode' ),
				'priority' => 10,
				'id'       => 'title_tagline',
			);

			// Manually add "Global" section because it is a default WP section (it doesn't exist in Kirki)
			$this->sections['background_image'] = array(
				'panel'    => 'grimlock_appearance_customizer_panel',
				'title'    => esc_html__( 'Global', 'grimlock' ),
				'priority' => 10,
				'id'       => 'background_image',
			);

			// Manually add "Header" section because it is a default WP section (it doesn't exist in Kirki)
			$this->sections['header_image'] = array(
				'panel'    => 'grimlock_appearance_customizer_panel',
				'title'    => esc_html__( 'Header', 'grimlock' ),
				'priority' => 60,
				'id'       => 'header_image',
			);

			// Generate Dark Mode sections
			foreach ( $this->sections as $section_id => $section ) {
				$section = new \Kirki\Section( 'grimlock_dark_mode_' . str_replace( 'grimlock_', '', $section_id ), array_merge( $section, array(
					'panel' => ! empty( $section['panel'] ) ? str_replace( 'grimlock_', 'grimlock_dark_mode_', $section['panel'] ) : 'grimlock_dark_mode_customizer_panel',
				) ) );
				$section->add_section( $wp_customize );
			}
		}
	}

	/**
	 * Generate dark mode specific customizer fields
	 */
	public function generate_dark_mode_fields() {
		if ( class_exists( 'Kirki' ) ) {

			remove_action( 'kirki_field_custom_init', array( $this, 'populate_fields' ), 10 );

			// Manually add "background_color" field because it is a default WP field (it doesn't exist in Kirki)
			$this->fields['background_color'] = array (
				'type'      => 'color',
				'field_class' => 'Kirki\Field\ReactColorful',
				'label'     => esc_html__( 'Background Color', 'grimlock' ),
				'section'   => 'background_image',
				'settings'  => 'background_color',
				'default'   => get_background_color(),
				'choices'   => array(
					'alpha'    => true,
					'palettes' => apply_filters( 'grimlock_color_field_palettes', array() ),
				),
				'priority'  => 10,
				'transport' => 'refresh',
				'output' => array(
					array(
						'element'  => 'body.custom-background',
						'property' => 'background-color',
					),
				)
			);

			// Manually add "custom_logo" field because it is a default WP field (it doesn't exist in Kirki)
			$this->fields['custom_logo'] = array (
				'type'      => 'image',
				'field_class' => 'Kirki\Field\Image',
				'label'     => esc_html__( 'Site logo', 'grimlock-dark-mode' ),
				'section'   => 'title_tagline',
				'settings'  => 'custom_logo',
				'priority'  => 10,
				'transport' => 'refresh',
				'output'    => array(),
			);

			// Rename fields that become ambiguous without tabs
			$fields_to_rename = array(
				'site_identity_blogname_color'       => esc_html__( 'Blog Name Color', 'grimlock-dark-mode' ),
				'site_identity_blogname_hover_color' => esc_html__( 'Blog Name Color on Hover', 'grimlock-dark-mode' ),

				'content_background_color' => esc_html__( 'Content Background Color', 'grimlock-dark-mode' ),

				'blockquote_color'            => esc_html__( 'Blockquote Color', 'grimlock-dark-mode' ),
				'blockquote_background_color' => esc_html__( 'Blockquote Background Color', 'grimlock-dark-mode' ),
				'blockquote_icon_color'       => esc_html__( 'Blockquote Icon Color', 'grimlock-dark-mode' ),
				'blockquote_border_color'     => esc_html__( 'Blockquote Border Color', 'grimlock-dark-mode' ),

				'button_primary_background_color'       => esc_html__( 'Primary Button Background Color', 'grimlock-dark-mode' ),
				'button_primary_color'                  => esc_html__( 'Primary Button Text Color', 'grimlock-dark-mode' ),
				'button_primary_border_color'           => esc_html__( 'Primary Button Border Color', 'grimlock-dark-mode' ),
				'button_primary_hover_background_color' => esc_html__( 'Primary Button Background Color on Hover', 'grimlock-dark-mode' ),
				'button_primary_hover_color'            => esc_html__( 'Primary Button Text Color on Hover', 'grimlock-dark-mode' ),
				'button_primary_hover_border_color'     => esc_html__( 'Primary Button Border Color on Hover', 'grimlock-dark-mode' ),

				'button_secondary_background_color'       => esc_html__( 'Secondary Button Background Color', 'grimlock-dark-mode' ),
				'button_secondary_color'                  => esc_html__( 'Secondary Button Text Color', 'grimlock-dark-mode' ),
				'button_secondary_border_color'           => esc_html__( 'Secondary Button Border Color', 'grimlock-dark-mode' ),
				'button_secondary_hover_background_color' => esc_html__( 'Secondary Button Background Color on Hover', 'grimlock-dark-mode' ),
				'button_secondary_hover_color'            => esc_html__( 'Secondary Button Text Color on Hover', 'grimlock-dark-mode' ),
				'button_secondary_hover_border_color'     => esc_html__( 'Secondary Button Border Color on Hover', 'grimlock-dark-mode' ),
			);

			// Generate Dark Mode fields
			foreach ( $this->fields as $field ) {

				// Skip if not a color field, or field has no output
				if (
					$field['settings'] !== 'custom_logo' &&
					(
						empty( $field['field_class'] ) ||
						( 'Kirki\Field\Color' !== $field['field_class'] && 'Kirki\Field\ReactColorful' !== $field['field_class'] ) ||
						empty( $field['output'] ) ||
						! is_array( $field['output'] )
					)
				) {
					continue;
				}

				$new_args = array_merge( $field, array(
					'section'  => 'grimlock_dark_mode_' . str_replace( 'grimlock_', '', $field['section'] ),
					'label'    => isset( $fields_to_rename[ $field['settings'] ] ) ? $fields_to_rename[ $field['settings'] ] : $field['label'],
					'settings' => "dark_mode_{$field['settings']}",
					'default'  => isset( $this->defaults[ "dark_mode_{$field['settings']}" ] ) ? $this->defaults[ "dark_mode_{$field['settings']}" ] : get_theme_mod( $field['settings'], $field['default'] ),
				) );

				if ( ! empty( $field['output'] ) && is_array( $field['output'] ) ) {
					foreach ( $new_args['output'] as $key => $output ) {
						// Skip if output has no element
						if ( empty( $output['element'] ) ) {
							continue;
						}

						$elements = is_array( $output['element'] ) ? $output['element'] : explode( ',', $output['element'] );

						// Prefix output elements with dark mode body class
						$new_args['output'][ $key ]['element'] = array_map( array(
							$this,
							'get_prefixed_element'
						), $elements );
					}
				}

				if ( ! empty( $field['js_vars'] ) && is_array( $field['js_vars'] ) ) {
					foreach ( $new_args['js_vars'] as $key => $output ) {
						// Skip if output has no element
						if ( empty( $output['element'] ) ) {
							continue;
						}

						$elements = is_array( $output['element'] ) ? $output['element'] : explode( ',', $output['element'] );

						$new_elements = array_map( array( $this, 'get_prefixed_element' ), $elements );

						$new_args['js_vars'][ $key ]['element'] = implode( ',', $new_elements );

						// Special case to enforce dark mode styles on regions
						if ( in_array( $field['section'], array( 'header_image', 'grimlock_404_customizer_section' ) ) ) {
							$new_args['output'][] = array_merge( $new_args['js_vars'][ $key ], array(
								'suffix'        => '!important',
								'value_pattern' => ! isset( $new_args['js_vars'][ $key ]['value_pattern'] ) || $new_args['js_vars'][ $key ]['value_pattern'] === '$ !important' ? '' : $new_args['js_vars'][ $key ]['value_pattern'],
							) );
						}
					}
				}

				if ( class_exists( $field['field_class'] ) ) {
					new $field['field_class']( array_merge( $field, $new_args ) );
				}
			}
		}
	}

	/**
	 * Generate Dark Mode fields based on existing Kirki fields
	 */
	public function add_customizer_fields() {
		$this->defaults = apply_filters( 'grimlock_dark_mode_customizer_defaults', array(
			'dark_mode_custom_logo'                   => ! empty( get_theme_mod( 'custom_logo' ) ) ? wp_get_attachment_image_url( get_theme_mod( 'custom_logo' ) ) : '',
			'dark_mode_toggle_button_location'        => 'bottom-left',
			'dark_mode_toggle_button_tooltip_enabled' => false,
			'dark_mode_toggle_button_icon'            => 'sun-moon',
			'dark_mode_default_mode'                  => 'light',
		) );

		$this->generate_dark_mode_fields();

		$this->add_toggle_button_location_field(        array( 'priority' => 10 ) );
		$this->add_toggle_button_tooltip_enabled_field( array( 'priority' => 20 ) );
		$this->add_toggle_button_icon_field(            array( 'priority' => 30 ) );
		$this->add_default_mode_field(                  array( 'priority' => 40 ) );
	}

	/**
	 * Add a Kirki select control to set the dark mode toggle button location in the Customizer.
	 *
	 * @param array $args
	 */
	protected function add_toggle_button_location_field( $args = array() ) {
		if ( class_exists( 'Kirki' ) ) {
			$args = wp_parse_args( $args, array(
				'type'     => 'select',
				'section'  => $this->section,
				'label'    => esc_html__( 'Dark mode toggle button location', 'grimlock-dark-mode' ),
				'settings' => 'dark_mode_toggle_button_location',
				'default'  => $this->get_default( 'dark_mode_toggle_button_location' ),
				'priority' => 10,
				'choices'  => array(
					'none'         => esc_html__( 'Hidden', 'grimlock-dark-mode' ),
					'bottom-left'  => esc_html__( 'Bottom left', 'grimlock-dark-mode' ),
					'bottom-right' => esc_html__( 'Bottom right', 'grimlock-dark-mode' ),
					'navigation'   => esc_html__( 'Navigation', 'grimlock-dark-mode' ),
				),
			) );

			Kirki::add_field( 'grimlock', apply_filters( 'grimlock_dark_mode_customizer_toggle_button_location_field_args', $args ) );
		}
	}

	/**
	 * Add a Kirki checkbox field to enabled a tooltip on the dark mode toggle button
	 *
	 * @param array $args
	 */
	protected function add_toggle_button_tooltip_enabled_field( $args = array() ) {
		if ( class_exists( 'Kirki') ) {
			$args = wp_parse_args( $args, array(
				'type'     => 'checkbox',
				'section'  => $this->section,
				'label'    => esc_html__( 'Enable the tooltip for the Dark Mode toggle button', 'grimlock-dark-mode' ),
				'settings' => 'dark_mode_toggle_button_tooltip_enabled',
				'default'  => $this->get_default( 'dark_mode_toggle_button_tooltip_enabled' ),
				'priority' => 10,
				'active_callback' => array(
					array(
						'setting'  => 'dark_mode_toggle_button_location',
						'operator' => '==',
						'value'    => 'navigation',
					),
				),
			) );

			Kirki::add_field( 'grimlock', apply_filters( 'grimlock_dark_mode_customizer_toggle_button_tooltip_enabled_field_args', $args ) );
		}
	}

	/**
	 * Add a Kirki radio-image field to set the icon for the dark mode toggle button in the Customizer.
	 *
	 * @param array $args
	 */
	protected function add_toggle_button_icon_field( $args = array() ) {
		if ( class_exists( 'Kirki' ) ) {
			$args = wp_parse_args( $args, array(
				'type'      => 'radio-buttonset',
				'section'   => $this->section,
				'label'     => esc_html__( 'Dark mode toggle button icon', 'grimlock-dark-mode' ),
				'settings'  => 'dark_mode_toggle_button_icon',
				'default'   => $this->get_default( 'dark_mode_toggle_button_icon' ),
				'priority'  => 10,
				'choices'   => array(
					'sun-moon'  => '<i class="fa fa-sun-o fa-2x" style="border-right: 1px solid; margin-right: 0.2em; padding-right: 0.2em;"></i><i class="fa fa-moon-o fa-2x"></i>',
					'lightbulb' => '<i class="fa fa-lightbulb-o fa-2x"></i>',
					'adjust'    => '<i class="fa fa-adjust fa-2x"></i>',
				),
			) );

			Kirki::add_field( 'grimlock', apply_filters( 'grimlock_dark_mode_customizer_toggle_button_icon_field_args', $args ) );
		}
	}

	/**
	 * Add a Kirki select control to set the dark mode toggle button location in the Customizer.
	 *
	 * @param array $args
	 */
	protected function add_default_mode_field( $args = array() ) {
		if ( class_exists( 'Kirki' ) ) {
			$args = wp_parse_args( $args, array(
				'type'     => 'select',
				'section'  => $this->section,
				'label'    => esc_html__( 'Default mode', 'grimlock-dark-mode' ),
				'description' => esc_html__( "Select which mode to use by default. Autodetect means the site will default to dark mode if the user's device is in dark mode.", 'grimlock-dark-mode' ),
				'settings' => 'dark_mode_default_mode',
				'default'  => $this->get_default( 'dark_mode_default_mode' ),
				'priority' => 10,
				'choices' => array(
					'light'      => esc_html__( 'Light', 'grimlock-dark-mode' ),
					'dark'       => esc_html__( 'Dark', 'grimlock-dark-mode' ),
					'autodetect' => esc_html__( 'Autodetect', 'grimlock-dark-mode' ),
				),
			) );

			Kirki::add_field( 'grimlock', apply_filters( 'grimlock_dark_mode_customizer_default_mode_field_args', $args ) );
		}
	}

	/**
	 * Returns the given CSS selector prefixed with the dark mode body class
	 *
	 * @param string $element The CSS selector to prefix
	 *
	 * @return string The prefixed CSS selector
	 */
	protected function get_prefixed_element( $element = '' ) {
		if ( empty( $element ) ) {
			return '';
		}

		if ( $element === ':root' ) {
			return '.grimlock-dark-mode--on';
		}

		if ( substr( $element, 0, 4 ) === 'body' ) {
			return 'body.grimlock-dark-mode--on' . substr( $element, 4 );
		}

		if ( $element[0] === '.' ) {
			// If first thing in the selector is a class, it could be a body class so we add two selectors
			return '.grimlock-dark-mode--on' . $element . ', .grimlock-dark-mode--on ' . $element;
		}

		return '.grimlock-dark-mode--on ' . $element;
	}

	/**
	 * Add body classes
	 *
	 * @param array $classes Array of body classes
	 *
	 * @return array
	 */
	public function add_body_class( $classes ) {
		if ( ! empty( $_COOKIE['grimlock_dark_mode'] ) || ( ! isset( $_COOKIE['grimlock_dark_mode'] ) && 'dark' === $this->get_theme_mod( 'dark_mode_default_mode' ) ) ) {
			$classes[] = 'grimlock-dark-mode--on';
		}

		$icon = $this->get_theme_mod( 'dark_mode_toggle_button_icon' );

		$classes[] = 'grimlock-dark-mode__toggle-icon--' . $icon;

		return $classes;
	}

	/**
	 * Display dark mode toggle button
	 */
	public function display_dark_mode_toggle() {
		$location = $this->get_theme_mod( 'dark_mode_toggle_button_location' );

		if ( in_array( $location, array( 'navigation', 'none' ) ) ) {
			return;
		}

		?>
		<button class="btn btn-secondary grimlock-dark-mode__toggle grimlock-dark-mode__toggle--<?php echo esc_attr( $location ); ?>"></button>
		<?php
	}

	/**
	 * Display dark mode toggle button in navigation
	 */
	public function navigation_display_dark_mode_toggle() {
		$location = $this->get_theme_mod( 'dark_mode_toggle_button_location' );
		$tooltips_enabled = $this->get_theme_mod( 'dark_mode_toggle_button_tooltip_enabled' );

		if ( $location !== 'navigation' || $location === 'none' ) {
			return;
		}

		?>
		<ul class="nav navbar-nav grimlock-navbar-nav--dark-mode-toggle">
			<li class="grimlock-menu-item menu-item">
				<button class="btn btn-secondary grimlock-dark-mode__toggle grimlock-dark-mode__toggle--<?php echo esc_attr( $location ); ?>" <?php if ( $tooltips_enabled ) : ?>data-toggle="tooltip" data-placement="bottom" title="<?php esc_attr_e( 'Toggle Light/Dark mode', 'grimlock-dark-mode' ); ?>"<?php endif; ?>></button>
			</li>
		</ul>
		<?php
	}

	/**
	 * Returns the default mode to use
	 *
	 * @param string $default The mode to use by default
	 *
	 * @return string
	 */
	public function default_mode( $default ) {
		// Back compat
		$legacy_autodetect_enabled = get_theme_mod( 'dark_mode_autodetect_enabled' );
		if ( empty( get_theme_mod( 'dark_mode_default_mode' ) ) && ! empty( $legacy_autodetect_enabled ) ) {
			return 'autodetect';
		}

		return $this->get_theme_mod( 'dark_mode_default_mode' );
	}

	public function change_site_identity_args( $args ) {
		if ( ! empty( $this->get_theme_mod( 'dark_mode_custom_logo' ) ) ) {
			$dark_mode_logo = '<a href="' . esc_url( home_url( '/' ) ) . '" rel="home" class="custom-logo-link grimlock-dark-mode-logo"><img src="' . esc_url( $this->get_theme_mod( 'dark_mode_custom_logo' ) ) . '" alt="logo" /></a>';

			$args['custom_logo'] = ! empty( $args['custom_logo'] ) ? $dark_mode_logo . $args['custom_logo'] : $dark_mode_logo;
		}

		return $args;
	}
}

return new Grimlock_Dark_Mode_Customizer();
